from typing import Optional

from fetchfox.checks import check_str
from .common import get


def get_average_price(asset_id: str, partner_code: str = None) -> Optional[float]:
    check_str(asset_id, "dexhunterio.asset_id")

    response = get(
        service=f"swap/averagePrice/{asset_id}/ADA",
        partner_code=partner_code,
    )

    if response is None:
        return None

    return response.get("price_ba")



import logging
import os
import time
from typing import Tuple

from fetchfox import rest

FREE = "free"
DEMO = "demo"
PRO = "pro"

API_KEY = os.getenv("COINGECKO_API_KEY") or os.getenv("COINGECKOCOM_API_KEY")
API_KEY_TYPE = os.getenv("COINGECKO_API_KEY_TYPE")

if API_KEY_TYPE is None:
    if API_KEY:
        API_KEY_TYPE = DEMO
    else:
        API_KEY_TYPE = FREE


if API_KEY_TYPE in PRO:
    BASE_URL = "https://pro-api.coingecko.com/api"
    RATE_LIMIT = 500
    HEADERS = {
        "x-cg-pro-api-key": API_KEY,
    }
elif API_KEY_TYPE in DEMO:
    BASE_URL = "https://api.coingecko.com/api"
    RATE_LIMIT = 30
    HEADERS = {
        "x-cg-demo-api-key": API_KEY,
    }
else:
    BASE_URL = "https://api.coingecko.com/api"
    RATE_LIMIT = 5
    HEADERS = {}


logger = logging.getLogger(__name__)


def get(service: str, params: dict = None, version: int = 3) -> Tuple[dict, int]:
    logger.debug("calling %s [%s]", service, API_KEY_TYPE)

    time.sleep(60 / RATE_LIMIT)

    return rest.get(
        url=f"{BASE_URL}/v{version}/{service}",
        headers=HEADERS,
        params=params,
    )



from typing import Optional, List

from pydantic import field_validator

from fetchfox.apis.base import BaseModel


class PolicyListing(BaseModel):
    class BundledAsset(BaseModel):
        asset_id: str
        display_name: str

    asset_id: str
    confirmed_at: str
    display_name: str
    tx_hash: str
    listing_id: int
    listed_at: str
    price_lovelace: int
    listing_type: str
    bundled_assets: Optional[List[BundledAsset]] = None

    @field_validator("price_lovelace", mode="before")
    def convert_price_lovelace(cls, value):
        return int(value)

    @property
    def url(self) -> str:
        return f"https://jpg.store/asset/{self.asset_id}"


class CollectionSale(BaseModel):
    class ListingFromTxHistory(BaseModel):
        class BundledAsset(BaseModel):
            asset_id: str
            display_name: str

        bundled_assets: Optional[List[BundledAsset]] = None

    tx_hash: str
    asset_id: str
    display_name: str
    bulk_size: int = 1
    action: str
    seller_address: str
    signer_address: str
    amount_lovelace: int
    confirmed_at: Optional[str] = None
    created_at: str

    listing_from_tx_history: Optional[ListingFromTxHistory] = None

    @field_validator("amount_lovelace", mode="before")
    def convert_amount_lovelace(cls, value):
        return int(value)

    @field_validator("bulk_size", mode="before")
    def ensure_bulk_size(cls, value):
        if value is None:
            return 1

        return int(value)

    @property
    def url(self) -> str:
        return f"https://jpg.store/asset/{self.asset_id}"



from typing import List


class TransactionAssetDTO:
    def __init__(self, amount: float, unit: str):
        self.amount: float = amount
        self.unit: str = unit

    def __repr__(self) -> str:
        return f"{self.amount} {self.unit}"


class TransactionInputOutputDTO:
    def __init__(self, address: str, assets: List[TransactionAssetDTO]):
        self.address: str = address
        self.assets: List[TransactionAssetDTO] = assets

    def __repr__(self) -> str:
        string = f"{self.address}: "

        for asset in self.assets:
            string += f"\n - {asset}"

        return string


class TransactionDTO:
    def __init__(
        self,
        blockchain: str,
        address: str,
        tx_hash: str,
        inputs: List[TransactionInputOutputDTO],
        outputs: List[TransactionInputOutputDTO],
        message: str = None,
    ):
        self.blockchain: str = blockchain
        self.address: str = address
        self.tx_hash: str = tx_hash
        self.inputs: List[TransactionInputOutputDTO] = inputs
        self.outputs: List[TransactionInputOutputDTO] = outputs
        self.message: str = message

    def __repr__(self) -> str:
        if not self.message:
            return self.tx_hash

        return f"{self.tx_hash} / {self.message}"



import logging
from functools import lru_cache

from fetchfox.checks import check_str
from .common import get

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def get_domain(address: str) -> str:
    check_str(address, "ensideascom.address")

    response, status_code = get(
        service=f"ens/resolve/{address}",
    )

    ens_domain = response.get("name")

    if not ens_domain:
        return None

    logger.info("resolved %s to %s", address, ens_domain)

    return ens_domain


@lru_cache(maxsize=None)
def resolve(ens_domain: str) -> str:
    check_str(ens_domain, "ensideascom.ens_domain")

    response, status_code = get(
        service=f"ens/resolve/{ens_domain}",
    )

    address = response.get("address")

    if not address:
        return None

    logger.info("resolved %s to %s", ens_domain, address)

    return address



BEARD_LP_MINSWAP_v1_ASSET_ID = (
    "e4214b7cce62ac6fbba385d164df48e157eae5863521b4b67ca71d862637dcfa955ec6eb059d12ecf3e038033fa856065e9eabe520b78cb1d337409c"
)
BEARD_LP_MINSWAP_v2_ASSET_ID = (
    "f5808c2c990d86da54bfc97d89cee6efa20cd8461616359478d96b4c52bd1e5a642e2141e8fb7ea6fa996960bc91db0b040c6d35013cf3efecac2dfc"
)
BOOK_LP_MINSWAP_v1_ASSET_ID = (
    "e4214b7cce62ac6fbba385d164df48e157eae5863521b4b67ca71d862ed309a7ecb6d0d5e00dca0bcc3924fdc0627a5fb631f1acc4deb898b14ee8bd"
)
BOOK_LP_MINSWAP_v2_ASSET_ID = (
    "f5808c2c990d86da54bfc97d89cee6efa20cd8461616359478d96b4cb68d128216cd07ee83667f1f5d2b65999b034161d88fada290eb5a9a6960c456"
)


LP_ASSETS = {
    BEARD_LP_MINSWAP_v1_ASSET_ID: "MINSWAP v1 $BEARD LP",
    BEARD_LP_MINSWAP_v2_ASSET_ID: "MINSWAP v2 $BEARD LP",
    BOOK_LP_MINSWAP_v1_ASSET_ID: "MINSWAP v1 $BOOK LP",
    BOOK_LP_MINSWAP_v2_ASSET_ID: "MINSWAP v2 $BOOK LP",
}



import logging
from functools import lru_cache
from typing import Optional

from fetchfox.checks import check_str
from .common import get

logger = logging.getLogger(__name__)


@lru_cache(maxsize=None)
def resolve(nf_domain: str) -> Optional[str]:
    check_str(nf_domain, "nfddomains.nf_domain")

    response, status_code = get(
        service=f"nfd/{nf_domain}",
    )

    if not response:
        return None

    address = response["owner"]
    logger.info("resolved %s to %s", nf_domain, address)

    return address


@lru_cache(maxsize=None)
def get_domain(address: str) -> Optional[str]:
    check_str(address, "nfddomains.address")
    address = address.strip().upper()

    response, status_code = get(
        service=f"nfd/v2/address",
        params={
            "address": address,
        },
    )

    if status_code == 404:
        return None

    nf_domains = sorted(
        set(map(lambda nfd: nfd["name"], response[address])),
        key=len,
    )

    if not nf_domains:
        return None

    nf_domain = nf_domains[0]
    logger.info("resolved %s to %s", address, nf_domain)

    return nf_domain



import logging
from typing import Iterable

from fetchfox.checks import check_str
from . import collections
from .common import get
from .schemas import EventsSalesSchema

logger = logging.getLogger(__name__)


def get_sales(contract_address: str, blockchain: str, slug: str = None, api_key: str = None) -> Iterable[EventsSalesSchema]:
    check_str(contract_address, "openseaio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(blockchain, "openseaio.blockchain")

    if not slug:
        slug = collections.get_slug(
            contract_address,
            blockchain,
            api_key=api_key,
        )

    cursor = ""

    while True:
        response, status_code = get(
            service=f"events/collection/{slug}",
            params={
                "event_type": "sale",
                "next": cursor,
            },
            api_key=api_key,
        )

        for event in response["asset_events"]:
            if event["event_type"] != "sale":
                continue

            yield EventsSalesSchema.model_validate(event)

        cursor = response.get("next")

        if not cursor:
            break



import base64
from functools import lru_cache

from fetchfox.checks import check_str
from fetchfox.constants.cardano.policies import ADA_HANDLE_POLICY_ID
from . import accounts
from . import assets


@lru_cache(maxsize=None)
def get_handle(stake_address: str, api_key: str = None, preprod: bool = False) -> str:
    check_str(stake_address, "gomaestroorg.stake_address")

    holdings = accounts.get_assets(
        stake_address,
        api_key=api_key,
        policy_id=ADA_HANDLE_POLICY_ID,
        preprod=preprod,
    )

    handles = []

    for holding in holdings:
        asset_id = holding.unit
        asset_id = asset_id.replace(f"{ADA_HANDLE_POLICY_ID}000de140", "")  # CIP-68
        asset_id = asset_id.replace(ADA_HANDLE_POLICY_ID, "")  # CIP-25

        asset_name = bytes.fromhex(asset_id).decode()
        handles.append(asset_name)

    if not handles:
        return None

    return sorted(handles, key=len)[0]


@lru_cache(maxsize=None)
def resolve_handle(handle: str, api_key: str = None, preprod: bool = False) -> str:
    check_str(handle, "gomaestroorg.handle")

    if handle.startswith("$"):
        if handle.startswith("$"):
            handle = handle[1:]

    handle = handle.lower()

    wallet = resolve_cip25_handle(handle, api_key, preprod=preprod)

    if wallet:
        return wallet

    return resolve_cip68_handle(handle, api_key, preprod=preprod)


def resolve_cip25_handle(handle: str, api_key: str = None, preprod: bool = False) -> str:
    check_str(handle, "gomaestroorg.handle")

    encoded_name = base64.b16encode(handle.encode()).decode("utf-8")

    asset_id = f"{ADA_HANDLE_POLICY_ID}{encoded_name}".lower()
    holders = list(assets.get_holders(asset_id, api_key, preprod=preprod))

    if not holders:
        return None

    return holders[0].account


def resolve_cip68_handle(handle: str, api_key: str = None, preprod: bool = False) -> str:
    check_str(handle, "gomaestroorg.handle")

    encoded_name = base64.b16encode(handle.encode()).decode("utf-8")

    asset_id = f"{ADA_HANDLE_POLICY_ID}000de140{encoded_name}".lower()
    holder = list(assets.get_holders(asset_id, api_key, preprod=preprod))

    if not holder:
        return None

    return holder[0].account



import json
from base64 import b64decode
from functools import lru_cache
from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import AssetsDataSchema, AssetHolderSchema


@lru_cache(maxsize=None)
def get_data(asset_id: str) -> AssetsDataSchema:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(
        service=f"assets/{asset_id}",
    )

    asset = response["asset"]
    return AssetsDataSchema.model_validate(asset)


@lru_cache(maxsize=None)
def get_metadata(asset_id: str) -> dict:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(
        f"assets/{asset_id}/transactions",
        params={
            "limit": "1",
            "tx-type": "acfg",
        },
    )

    transaction = response["transactions"][0]

    if "note" in transaction:
        note = b64decode(transaction["note"]).decode("utf-8")
    else:
        note = "{}"

    return json.loads(note)


def get_holders(asset_id: str) -> Iterable[AssetHolderSchema]:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(
        service=f"assets/{asset_id}/balances",
        params={
            "currency-greater-than": "0",
        },
    )

    for balance in response.get("balances", []):
        balance["asset-id"] = asset_id

        yield AssetHolderSchema.model_validate(balance)



from typing import List

from pydantic import Field

from fetchfox.apis.base import BaseModel


class CollectionsListingsSchema(BaseModel):
    class PriceSchema(BaseModel):
        class CurrentSchema(BaseModel):
            currency: str
            decimals: int
            value: float

        current: CurrentSchema

    class ProtocolDataSchema(BaseModel):
        class ParametersSchema(BaseModel):
            class OfferSchema(BaseModel):
                identifier_or_criteria: str = Field(..., alias="identifierOrCriteria")
                token: str

            offerer: str
            offer: List[OfferSchema]
            start_time: int = Field(..., alias="startTime")

        parameters: ParametersSchema

    order_hash: str
    price: PriceSchema
    protocol_data: ProtocolDataSchema


class EventsSalesSchema(BaseModel):
    class NftSchema(BaseModel):
        contract: str
        identifier: str
        name: str

    class PaymentSchema(BaseModel):
        decimals: int
        quantity: float
        symbol: str

    buyer: str
    closing_date: int
    seller: str
    transaction: str
    nft: NftSchema
    payment: PaymentSchema



ADA_HANDLE_POLICY_ID = "f0ff48bbb7bbe9d59a40f1ce90e9e9d0ff5002ec48f232b49ca0fb9a"
GUTENBERG_BIBLE_POLICY_ID = "477cec772adb1466b301fb8161f505aa66ed1ee8d69d3e7984256a43"
THE_ATTACK_ON_VITALIK_POLICY_ID = "58b2b7da117fdfe9c2afaf5da892eb44850c014a5a47243884e8aac4"



import json
from functools import lru_cache
from typing import Iterable

from fetchfox.apis import pinatacloud
from fetchfox.checks import check_str
from .common import get
from .schemas import AssetHoldersSchema


@lru_cache(maxsize=None)
def get_data(contract_address: str, asset_id: str, blockchain: str, api_key: str = None, preprod: bool = False) -> dict:
    check_str(contract_address, "moralisio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(asset_id, "moralisio.asset_id")

    response, status_code = get(
        service=f"nft/{contract_address}/{asset_id}",
        blockchain=blockchain,
        api_key=api_key,
        preprod=preprod,
    )

    if status_code == 404:
        raise Exception(f"{contract_address}/{asset_id} doesn't exist")

    if response.get("metadata"):  # metadata is cached by moralis
        metadata = json.loads(response["metadata"])
    else:
        # trigger metadata resync on moralis
        resync_metadata(contract_address, asset_id, blockchain, api_key)

        # fetch metadata from book.io's ipfs node
        token_uri = response["token_uri"].split("ipfs/")[-1]
        metadata = pinatacloud.get_metadata(token_uri)

    metadata["attributes"].update(metadata["extraAttributes"])
    response["metadata"] = metadata

    return response


def resync_metadata(contract_address: str, asset_id: str, blockchain: str, api_key: str = None, preprod: bool = False):
    check_str(contract_address, "moralisio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(asset_id, "moralisio.asset_id")

    response, status_code = get(
        service=f"nft/{contract_address}/{asset_id}/metadata/resync",
        params={
            "flag": "uri",
            "mode": "sync",
        },
        blockchain=blockchain,
        api_key=api_key,
        preprod=preprod,
    )

    if status_code == 404:
        raise ValueError(f"{contract_address}/{asset_id} doesn't exist")

    return response.get("status")


def get_holders(contract_address: str, asset_id: str, blockchain: str, api_key: str = None, preprod: bool = False) -> Iterable[AssetHoldersSchema]:
    check_str(contract_address, "moralisio.contract_address")
    contract_address = contract_address.strip().lower()

    check_str(asset_id, "moralisio.asset_id")

    response, status_code = get(
        service=f"nft/{contract_address}/{asset_id}/owners",
        blockchain=blockchain,
        api_key=api_key,
        preprod=preprod,
    )

    yield from map(
        AssetHoldersSchema.model_validate,
        response.get("result", []),
    )



import os

from fetchfox import rest

BASE_URL_DEFAULT = "https://book.mypinata.cloud/ipfs"
BASE_URL = os.getenv("PINATACLOUD_BASE_URL") or BASE_URL_DEFAULT


def get_metadata(uri: str) -> dict:
    uri = uri.replace("ipfs://", "")

    response, status_code = rest.get(
        url=f"{BASE_URL}/{uri}",
    )

    return response



from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import PolicyListing


def get_listings(policy_id: str) -> Iterable[PolicyListing]:
    check_str(policy_id, "jpgstore.policy_id")
    policy_id = policy_id.strip().lower()

    cursor = ""

    while True:
        params = {}

        if cursor:
            params["cursor"] = cursor

        response, status_code = get(
            service=f"policy/{policy_id}/listings",
            params=params,
        )

        cursor = response.get("nextPageCursor")

        if not cursor:  # pragma: no cover
            break

        yield from map(
            PolicyListing.model_validate,
            response["listings"],
        )



from typing import Iterable

from fetchfox.checks import check_str
from .common import get
from .schemas import CollectionSale


def get_sales(policy_id: str) -> Iterable[CollectionSale]:
    check_str(policy_id, "jpgstore.policy_id")
    policy_id = policy_id.strip().lower()

    txs = set()

    last_date = ""

    while True:
        response, status_code = get(
            service=f"collection/{policy_id}/v2/transactions",
            params={
                "lastDate": last_date,
                "count": 50,
            },
            headers={
                "x-jpgstore-csrf-protection": "1",
            },
        )

        transactions = response.get("transactions")

        if not transactions:  # pragma: no cover
            break

        for transaction in transactions:
            sale = CollectionSale.model_validate(transaction)

            if sale.tx_hash in txs:
                continue

            txs.add(sale.tx_hash)
            last_date = sale.created_at

            yield sale



import logging
from datetime import datetime
from typing import Iterable

from cachetools.func import ttl_cache

from fetchfox.constants.currencies import ALGO, ADA, BOOK, ETH, MATIC, POL, STUFF, USD
from .common import get
from .schemas import CoinExchangeHistorySchema

logger = logging.getLogger(__name__)

IDS = {
    ALGO: "algorand",
    ADA: "cardano",
    BOOK: "book-2",
    ETH: "ethereum",
    MATIC: "matic-network",
    POL: "polygon-ecosystem-token",
    STUFF: "book-2",
}


@ttl_cache(ttl=60 * 60)
def get_exchange(crypto: str, fiat: str = USD, date: datetime = None) -> float:
    crypto, fiat = crypto.strip().upper(), fiat.strip().lower()
    coin_id = IDS[crypto]

    logger.info("fetching exchange %s/%s (%s)", crypto, fiat, coin_id)

    if date is None:
        response, status_code = get(
            service="simple/price",
            params={
                "ids": coin_id,
                "vs_currencies": fiat,
            },
        )

        return response[coin_id][fiat]
    else:
        date = date or datetime.now()

        response, status_code = get(
            service=f"coins/{coin_id}/history",
            params={
                "date": date.strftime("%d-%m-%Y"),
                "localization": "false",
            },
        )

        market_data = response.get("market_data")

        if not market_data:
            return None

        return market_data["current_price"][fiat]


@ttl_cache(ttl=60 * 60)
def get_ath(crypto: str, fiat: str = USD) -> float:
    crypto, fiat = crypto.strip().upper(), fiat.strip().lower()
    coin_id = IDS[crypto]

    logger.info("fetching ath for %s/%s (%s)", crypto, fiat, coin_id)

    response, status_code = get(
        service=f"coins/{coin_id}",
    )

    return response["market_data"]["ath"][fiat]


def get_exchange_history(crypto: str, fiat: str = USD, days: int = 7) -> Iterable[CoinExchangeHistorySchema]:
    crypto, fiat = crypto.strip().upper(), fiat.strip().lower()
    coin_id = IDS[crypto]

    logger.info("fetching exchange history of %s/%s (%s)", crypto, fiat, coin_id)

    response, status_code = get(
        service=f"coins/{coin_id}/market_chart",
        params={
            "vs_currency": fiat,
            "days": days,
        },
    )

    for timestampe, price in response["prices"]:
        yield CoinExchangeHistorySchema(
            crypto=crypto,
            fiat=fiat,
            timestamp=timestampe,
            price=price,
        )



def check_str(string: str, name: str = None) -> bool:
    if not name:
        name = "value"

    if string is None:
        raise ValueError(f"{name} is none")

    if not isinstance(string, str):
        raise ValueError(f"{name} is not a string")

    if len(string) == 0:
        raise ValueError(f"{name} is empty")



from fetchfox.constants.cardano.tokens.charles import (
    CHARLES_TOKEN_ASSET_ID,
    CHARLES_TOKEN_ASSET_NAME,
    CHARLES_TOKEN_POLICY_ID,
    CHARLES_TOKEN_FINGERPRINT,
    CHARLES_TOKEN_SYMBOL,
)

from .base import CardanoToken


class CharlesToken(CardanoToken):
    def __init__(self, dexhunterio_partner_code: str = None):
        super().__init__(
            asset_id=CHARLES_TOKEN_ASSET_ID,
            asset_name=CHARLES_TOKEN_ASSET_NAME,
            fingerprint=CHARLES_TOKEN_FINGERPRINT,
            policy_id=CHARLES_TOKEN_POLICY_ID,
            symbol=CHARLES_TOKEN_SYMBOL,
            decimals=0,
            taptools_pair_id="bfcbf9a63822d7bc1f12297bfe625432a9486ebf4a05d4925fa158cf8c352a8f",
            dexhunterio_partner_code=dexhunterio_partner_code,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fetchfox',
 'fetchfox.apis',
 'fetchfox.apis.algorand',
 'fetchfox.apis.algorand.algonodecloud',
 'fetchfox.apis.algorand.nfdomains',
 'fetchfox.apis.algorand.randswapcom',
 'fetchfox.apis.cardano',
 'fetchfox.apis.cardano.dexhunterio',
 'fetchfox.apis.cardano.gomaestroorg',
 'fetchfox.apis.cardano.jpgstore',
 'fetchfox.apis.coingeckocom',
 'fetchfox.apis.evm',
 'fetchfox.apis.evm.ensideascom',
 'fetchfox.apis.evm.moralisio',
 'fetchfox.apis.evm.openseaio',
 'fetchfox.blockchains',
 'fetchfox.blockchains.algorand',
 'fetchfox.blockchains.cardano',
 'fetchfox.blockchains.ethereum',
 'fetchfox.blockchains.evm',
 'fetchfox.blockchains.polygon',
 'fetchfox.constants',
 'fetchfox.constants.algorand',
 'fetchfox.constants.cardano',
 'fetchfox.constants.cardano.pools',
 'fetchfox.constants.cardano.tokens',
 'fetchfox.constants.ethereum',
 'fetchfox.constants.evm',
 'fetchfox.constants.polygon',
 'fetchfox.dtos',
 'fetchfox.helpers',
 'fetchfox.pools',
 'fetchfox.tokens']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.4,<4.0.0',
 'asyncio>=3.4.3,<4.0.0',
 'backoff>=2.2.1,<3.0.0',
 'cachetools>=5.3.1,<6.0.0',
 'certifi>=2024.2.2,<2025.0.0',
 'pydantic>=2.10.3,<3.0.0',
 'pytz>=2024.2,<2025.0']

setup_kwargs = {
    'name': 'bookio-fetchfox',
    'version': '5.1.0',
    'description': 'Collection of API services to fetch information from several blockchains.',
    'long_description': '# book.io / fetchfox\n\n> Collection of API services to fetch information from several blockchains.\n\n![](https://s2.coinmarketcap.com/static/img/coins/64x64/4030.png)\n![](https://s2.coinmarketcap.com/static/img/coins/64x64/2010.png)\n![](https://s2.coinmarketcap.com/static/img/coins/64x64/1027.png)\n![](https://s2.coinmarketcap.com/static/img/coins/64x64/3890.png)\n\n\n## Supported Blockchains\n\n### Algorand\n\n```python\nimport os\nfrom fetchfox.blockchains import Algorand\n\nalgorand = Algorand()\n\n# Brave New World\ncreator_address = "6WII6ES4H6UW7G7T7RJX63CUNPKJEPEGQ3PTYVVU3JHJ652W34GCJV5OVY"\n\nfor asset in algorand.get_assets(creator_address):\n    print(asset)\n```\n\n#### Services\n\n* get_asset ([algonode.cloud](https://algonode.cloud))\n* get_assets ([algonode.cloud](https://algonode.cloud))\n* get_holdings ([algonode.cloud](https://algonode.cloud))\n* get_snapshot ([algonode.cloud](https://algonode.cloud))\n* get_listings ([randgallery.com](https://randgallery.com) / [algoxnft.com](https://algoxnft.com))\n* get_floor ([randgallery.com](https://randgallery.com) / [algoxnft.com](https://algoxnft.com))\n* get_sales ([nftexplorer.app¹](https://nftexplorer.app))\n\n\n### Cardano\n\n```python\nimport os\nfrom fetchfox.blockchains import Cardano\n\ncardano = Cardano(\n    gomaestroorg_api_key=os.getenv("GOMAESTROORG_API_KEY"),\n)\n\n# Gutenberg Bible\npolicy_id = "477cec772adb1466b301fb8161f505aa66ed1ee8d69d3e7984256a43"\n\nfor asset in cardano.get_collection_assets(policy_id):\n    print(asset)\n```\n\n#### Services\n\n* get_asset ([gomaestro.org²](https://gomaestro.org))\n* get_assets ([gomaestro.org²](https://gomaestro.org))\n* get_holdings ([gomaestro.org²](https://bgomaestro.org))\n* get_snapshot ([gomaestro.org²](https://gomaestro.org))\n* get_listings ([jpg.store](https://jpg.store))\n* get_floor ([jpg.store](https://jpg.store))\n* get_sales ([jpg.store](https://jpg.store))\n\n\n### EVM (Ethereum and Polygon)\n\n```python\nimport os\nfrom fetchfox.blockchains import Ethereum, Polygon\n\nethereum = Ethereum(\n    geckodriver_path=os.getenv("GECKODRIVER_PATH"),\n    moralisio_api_key=os.getenv("MORALIS_API_KEY"),\n    openseaio_api_key=os.getenv("OPENSEA_API_KEY"),\n)\n\npolygon = Polygon(\n    geckodriver_path=os.getenv("GECKODRIVER_PATH"),\n    moralisio_api_key=os.getenv("MORALIS_API_KEY"),\n    openseaio_api_key=os.getenv("OPENSEA_API_KEY"),\n)\n\n\n# Alice in Wonderland\ncontract_address = "0x919da7fef646226f88f70305201de392ff365059"\n\nfor asset in ethereum.get_assets(contract_address):\n    print(asset)\n\n# Art of War\ncontract_address = "0xb56010e0500e4f163758881603b8083996ae47ec"\n\nfor asset in polygon.get_assets(contract_address):\n    print(asset)\n```\n\n#### Services\n\n* get_asset ([moralis.io³](https://moralis.io))\n* get_assets ([moralis.io³](https://moralis.io))\n* get_holdings ([moralis.io³](https://moralis.io))\n* get_snapshot ([moralis.io³](https://moralis.io))\n* get_listings ([opensea.io⁴](https://opensea.io))\n* get_floor ([opensea.io⁴](https://opensea.io))\n* get_sales ([opensea.io⁴](https://opensea.io))\n\n\n> ¹ **nftexplorer.app** this api has been deprecated.\n> \n> ² **gomaestro.org** services require an [api key](https://www.gomaestro.org/pricing).\n> \n> ³ **moralis.io** services require an [api key](https://moralis.io/pricing).\n> \n> ⁴ **opensea.io** some services also require an [api key](https://docs.opensea.io/reference/api-keys). \n\n---\n\n![fetch, the fox](https://i.imgur.com/fm6mqzS.png)\n> fetch, the fox\n\n',
    'author': 'Fede',
    'author_email': 'fede@book.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/book-io/fetchfox',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Optional, List

from pydantic import field_validator, Field

from fetchfox.apis.base import BaseModel


class AccountAssetSchema(BaseModel):
    unit: str
    amount: int


class AssetHolderSchema(BaseModel):
    asset_id: str
    account: str
    amount: int

    @field_validator("amount", mode="before")
    def convert_amount(cls, value):
        return int(value)


class PolicyHolderSchema(BaseModel):
    class AccountAssetSchema(BaseModel):
        name: str
        amount: int

        @field_validator("amount", mode="before")
        def convert_amount(cls, value):
            return int(value)

    account: str
    assets: List[AccountAssetSchema]


class CollectionAssetSchema(BaseModel):
    class AssetStandardsSchema(BaseModel):
        cip25_metadata: Optional[dict]
        cip68_metadata: Optional[dict]

    asset_name: str
    total_supply: int

    asset_standards: AssetStandardsSchema

    @field_validator("total_supply", mode="before")
    def convert_total_supply(cls, value):
        return int(value)


class TransactionSchema(BaseModel):
    class InputOutputSchema(BaseModel):
        class AssetSchema(BaseModel):
            unit: str
            amount: int

            @field_validator("amount", mode="before")
            def convert_amount(cls, value):
                return int(value)

        address: str
        assets: List[AssetSchema] = []

    class MetadataSchema(BaseModel):
        class M674Schema(BaseModel):
            msg: Optional[str] = None

            @field_validator("msg", mode="before")
            def ensure_string(cls, value):
                if value is None:
                    return None

                if isinstance(value, list):
                    return " ".join(value)

                return str(value)

        m_674: Optional[M674Schema] = Field(None, alias="674")

    tx_hash: str
    block_timestamp: int
    fee: int

    inputs: List[InputOutputSchema]
    outputs: List[InputOutputSchema]
    metadata: Optional[MetadataSchema] = None


class PoolInformationSchema(BaseModel):
    live_saturation: float
    live_stake: float
    live_delegators: float

    @field_validator("live_saturation", mode="before")
    def convert_live_saturation(cls, value):
        return float(value)

    @field_validator("live_stake", mode="before")
    def convert_live_stake(cls, value):
        return float(value)

    @field_validator("live_delegators", mode="before")
    def convert_live_delegators(cls, value):
        return float(value)


class PoolDelegatorSchema(BaseModel):
    stake_address: str
    amount: float

    @field_validator("amount", mode="before")
    def convert_amount(cls, value):
        return float(value)
